@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,42 +18,12 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../../SDL_internal.h"
+#include "SDL_internal.h"
 
 #ifdef SDL_TIMER_N3DS
 
 #include <3ds.h>
 
-static SDL_bool ticks_started = SDL_FALSE;
-static u64 start_tick;
-
-#define NSEC_PER_MSEC 1000000ULL
-
-void SDL_TicksInit(void)
-{
-    if (ticks_started) {
-        return;
-    }
-    ticks_started = SDL_TRUE;
-
-    start_tick = svcGetSystemTick();
-}
-
-void SDL_TicksQuit(void)
-{
-    ticks_started = SDL_FALSE;
-}
-
-Uint64 SDL_GetTicks64(void)
-{
-    u64 elapsed;
-    if (!ticks_started) {
-        SDL_TicksInit();
-    }
-
-    elapsed = svcGetSystemTick() - start_tick;
-    return elapsed / CPU_TICKS_PER_MSEC;
-}
 
 Uint64 SDL_GetPerformanceCounter(void)
 {
@@ -65,11 +35,9 @@ Uint64 SDL_GetPerformanceFrequency(void)
     return SYSCLOCK_ARM11;
 }
 
-void SDL_Delay(Uint32 ms)
+void SDL_SYS_DelayNS(Uint64 ns)
 {
-    svcSleepThread(ms * NSEC_PER_MSEC);
+    svcSleepThread(ns);
 }
 
-#endif /* SDL_TIMER_N3DS */
-
-/* vi: set sts=4 ts=4 sw=4 expandtab: */
+#endif // SDL_TIMER_N3DS
