@@ -1,6 +1,6 @@
 /*
   Simple DirectMedia Layer
-  Copyright (C) 1997-2023 Sam Lantinga <slouken@libsdl.org>
+  Copyright (C) 1997-2025 Sam Lantinga <slouken@libsdl.org>
 
   This software is provided 'as-is', without any express or implied
   warranty.  In no event will the authors be held liable for any damages
@@ -18,16 +18,14 @@
      misrepresented as being the original software.
   3. This notice may not be removed or altered from any source distribution.
 */
-#include "../SDL_internal.h"
+#include "SDL_internal.h"
 
-#if SDL_HAVE_BLIT_1
+#ifdef SDL_HAVE_BLIT_1
 
-#include "SDL_video.h"
-#include "SDL_blit.h"
+#include "SDL_surface_c.h"
 #include "SDL_sysvideo.h"
-#include "SDL_endian.h"
 
-/* Functions to blit from 8-bit surfaces to other surfaces */
+// Functions to blit from 8-bit surfaces to other surfaces
 
 static void Blit1to1(SDL_BlitInfo *info)
 {
@@ -38,7 +36,7 @@ static void Blit1to1(SDL_BlitInfo *info)
     Uint8 *src, *map, *dst;
     int srcskip, dstskip;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     width = info->dst_w;
     height = info->dst_h;
     src = info->src;
@@ -49,15 +47,15 @@ static void Blit1to1(SDL_BlitInfo *info)
 
     while (height--) {
 #ifdef USE_DUFFS_LOOP
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP_TRIVIAL(
             {
               *dst = map[*src];
             }
             dst++;
             src++;
         , width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
 #else
         for (c = width; c; --c) {
             *dst = map[*src];
@@ -70,12 +68,12 @@ static void Blit1to1(SDL_BlitInfo *info)
     }
 }
 
-/* This is now endian dependent */
+// This is now endian dependent
 #ifndef USE_DUFFS_LOOP
 #if (SDL_BYTEORDER == SDL_LIL_ENDIAN)
 #define HI 1
 #define LO 0
-#else /* ( SDL_BYTEORDER == SDL_BIG_ENDIAN ) */
+#else // ( SDL_BYTEORDER == SDL_BIG_ENDIAN )
 #define HI 0
 #define LO 1
 #endif
@@ -90,7 +88,7 @@ static void Blit1to2(SDL_BlitInfo *info)
     Uint16 *map;
     int srcskip, dstskip;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     width = info->dst_w;
     height = info->dst_h;
     src = info->src;
@@ -101,32 +99,32 @@ static void Blit1to2(SDL_BlitInfo *info)
 
 #ifdef USE_DUFFS_LOOP
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP_TRIVIAL(
         {
             *(Uint16 *)dst = map[*src++];
             dst += 2;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dst += dstskip;
     }
 #else
-    /* Memory align at 4-byte boundary, if necessary */
+    // Memory align at 4-byte boundary, if necessary
     if ((long)dst & 0x03) {
-        /* Don't do anything if width is 0 */
+        // Don't do anything if width is 0
         if (width == 0) {
             return;
         }
         --width;
 
         while (height--) {
-            /* Perform copy alignment */
+            // Perform copy alignment
             *(Uint16 *)dst = map[*src++];
             dst += 2;
 
-            /* Copy in 4 pixel chunks */
+            // Copy in 4 pixel chunks
             for (c = width / 4; c; --c) {
                 *(Uint32 *)dst = (map[src[HI]] << 16) | (map[src[LO]]);
                 src += 2;
@@ -135,7 +133,7 @@ static void Blit1to2(SDL_BlitInfo *info)
                 src += 2;
                 dst += 4;
             }
-            /* Get any leftovers */
+            // Get any leftovers
             switch (width & 3) {
             case 3:
                 *(Uint16 *)dst = map[*src++];
@@ -156,7 +154,7 @@ static void Blit1to2(SDL_BlitInfo *info)
         }
     } else {
         while (height--) {
-            /* Copy in 4 pixel chunks */
+            // Copy in 4 pixel chunks
             for (c = width / 4; c; --c) {
                 *(Uint32 *)dst = (map[src[HI]] << 16) | (map[src[LO]]);
                 src += 2;
@@ -165,7 +163,7 @@ static void Blit1to2(SDL_BlitInfo *info)
                 src += 2;
                 dst += 4;
             }
-            /* Get any leftovers */
+            // Get any leftovers
             switch (width & 3) {
             case 3:
                 *(Uint16 *)dst = map[*src++];
@@ -185,7 +183,7 @@ static void Blit1to2(SDL_BlitInfo *info)
             dst += dstskip;
         }
     }
-#endif /* USE_DUFFS_LOOP */
+#endif // USE_DUFFS_LOOP
 }
 
 static void Blit1to3(SDL_BlitInfo *info)
@@ -198,7 +196,7 @@ static void Blit1to3(SDL_BlitInfo *info)
     Uint8 *src, *map, *dst;
     int srcskip, dstskip;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     width = info->dst_w;
     height = info->dst_h;
     src = info->src;
@@ -209,7 +207,7 @@ static void Blit1to3(SDL_BlitInfo *info)
 
     while (height--) {
 #ifdef USE_DUFFS_LOOP
-        /* *INDENT-OFF* */ /* clang-format off */
+        /* *INDENT-OFF* */ // clang-format off
         DUFFS_LOOP(
             {
                 o = *src * 4;
@@ -220,7 +218,7 @@ static void Blit1to3(SDL_BlitInfo *info)
             src++;
             dst += 3;
         , width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
 #else
         for (c = width; c; --c) {
             o = *src * 4;
@@ -230,7 +228,7 @@ static void Blit1to3(SDL_BlitInfo *info)
             src++;
             dst += 3;
         }
-#endif /* USE_DUFFS_LOOP */
+#endif // USE_DUFFS_LOOP
         src += srcskip;
         dst += dstskip;
     }
@@ -246,7 +244,7 @@ static void Blit1to4(SDL_BlitInfo *info)
     Uint32 *map, *dst;
     int srcskip, dstskip;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     width = info->dst_w;
     height = info->dst_h;
     src = info->src;
@@ -257,11 +255,11 @@ static void Blit1to4(SDL_BlitInfo *info)
 
     while (height--) {
 #ifdef USE_DUFFS_LOOP
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP_TRIVIAL(
             *dst++ = map[*src++];
         , width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
 #else
         for (c = width / 4; c; --c) {
             *dst++ = map[*src++];
@@ -279,7 +277,7 @@ static void Blit1to4(SDL_BlitInfo *info)
         case 1:
             *dst++ = map[*src++];
         }
-#endif /* USE_DUFFS_LOOP */
+#endif // USE_DUFFS_LOOP
         src += srcskip;
         dst += dstskip;
     }
@@ -298,8 +296,8 @@ static void Blit1to1Key(SDL_BlitInfo *info)
 
     if (palmap) {
         while (height--) {
-            /* *INDENT-OFF* */ /* clang-format off */
-            DUFFS_LOOP(
+            /* *INDENT-OFF* */ // clang-format off
+            DUFFS_LOOP_TRIVIAL(
             {
                 if ( *src != ckey ) {
                   *dst = palmap[*src];
@@ -308,14 +306,14 @@ static void Blit1to1Key(SDL_BlitInfo *info)
                 src++;
             },
             width);
-            /* *INDENT-ON* */ /* clang-format on */
+            /* *INDENT-ON* */ // clang-format on
             src += srcskip;
             dst += dstskip;
         }
     } else {
         while (height--) {
-            /* *INDENT-OFF* */ /* clang-format off */
-            DUFFS_LOOP(
+            /* *INDENT-OFF* */ // clang-format off
+            DUFFS_LOOP_TRIVIAL(
             {
                 if ( *src != ckey ) {
                   *dst = *src;
@@ -324,7 +322,7 @@ static void Blit1to1Key(SDL_BlitInfo *info)
                 src++;
             },
             width);
-            /* *INDENT-ON* */ /* clang-format on */
+            /* *INDENT-ON* */ // clang-format on
             src += srcskip;
             dst += dstskip;
         }
@@ -342,12 +340,12 @@ static void Blit1to2Key(SDL_BlitInfo *info)
     Uint16 *palmap = (Uint16 *)info->table;
     Uint32 ckey = info->colorkey;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     dstskip /= 2;
 
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP_TRIVIAL(
         {
             if ( *src != ckey ) {
                 *dstp=palmap[*src];
@@ -356,7 +354,7 @@ static void Blit1to2Key(SDL_BlitInfo *info)
             dstp++;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dstp += dstskip;
     }
@@ -375,7 +373,7 @@ static void Blit1to3Key(SDL_BlitInfo *info)
     int o;
 
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
+        /* *INDENT-OFF* */ // clang-format off
         DUFFS_LOOP(
         {
             if ( *src != ckey ) {
@@ -388,7 +386,7 @@ static void Blit1to3Key(SDL_BlitInfo *info)
             dst += 3;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dst += dstskip;
     }
@@ -405,12 +403,12 @@ static void Blit1to4Key(SDL_BlitInfo *info)
     Uint32 *palmap = (Uint32 *)info->table;
     Uint32 ckey = info->colorkey;
 
-    /* Set up some basic variables */
+    // Set up some basic variables
     dstskip /= 4;
 
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP_TRIVIAL(
         {
             if ( *src != ckey ) {
                 *dstp = palmap[*src];
@@ -419,7 +417,7 @@ static void Blit1to4Key(SDL_BlitInfo *info)
             dstp++;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dstp += dstskip;
     }
@@ -433,32 +431,33 @@ static void Blit1toNAlpha(SDL_BlitInfo *info)
     int srcskip = info->src_skip;
     Uint8 *dst = info->dst;
     int dstskip = info->dst_skip;
-    SDL_PixelFormat *dstfmt = info->dst_fmt;
-    const SDL_Color *srcpal = info->src_fmt->palette->colors;
+    const SDL_PixelFormatDetails *dstfmt = info->dst_fmt;
+    const SDL_Color *srcpal = info->src_pal->colors;
     int dstbpp;
     Uint32 pixel;
-    unsigned sR, sG, sB;
+    unsigned sR, sG, sB, sA;
     unsigned dR, dG, dB, dA;
     const unsigned A = info->a;
 
-    /* Set up some basic variables */
-    dstbpp = dstfmt->BytesPerPixel;
+    // Set up some basic variables
+    dstbpp = dstfmt->bytes_per_pixel;
 
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
-        DUFFS_LOOP4(
+        /* *INDENT-OFF* */ // clang-format off
+        DUFFS_LOOP(
         {
             sR = srcpal[*src].r;
             sG = srcpal[*src].g;
             sB = srcpal[*src].b;
+            sA = (srcpal[*src].a * A) / 255;
             DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
-            ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
+            ALPHA_BLEND_RGBA(sR, sG, sB, sA, dR, dG, dB, dA);
             ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
             src++;
             dst += dstbpp;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dst += dstskip;
     }
@@ -472,35 +471,36 @@ static void Blit1toNAlphaKey(SDL_BlitInfo *info)
     int srcskip = info->src_skip;
     Uint8 *dst = info->dst;
     int dstskip = info->dst_skip;
-    SDL_PixelFormat *dstfmt = info->dst_fmt;
-    const SDL_Color *srcpal = info->src_fmt->palette->colors;
+    const SDL_PixelFormatDetails *dstfmt = info->dst_fmt;
+    const SDL_Color *srcpal = info->src_pal->colors;
     Uint32 ckey = info->colorkey;
     int dstbpp;
     Uint32 pixel;
-    unsigned sR, sG, sB;
+    unsigned sR, sG, sB, sA;
     unsigned dR, dG, dB, dA;
     const unsigned A = info->a;
 
-    /* Set up some basic variables */
-    dstbpp = dstfmt->BytesPerPixel;
+    // Set up some basic variables
+    dstbpp = dstfmt->bytes_per_pixel;
 
     while (height--) {
-        /* *INDENT-OFF* */ /* clang-format off */
+        /* *INDENT-OFF* */ // clang-format off
         DUFFS_LOOP(
         {
             if ( *src != ckey ) {
                 sR = srcpal[*src].r;
                 sG = srcpal[*src].g;
                 sB = srcpal[*src].b;
+                sA = (srcpal[*src].a * A) / 255;
                 DISEMBLE_RGBA(dst, dstbpp, dstfmt, pixel, dR, dG, dB, dA);
-                ALPHA_BLEND_RGBA(sR, sG, sB, A, dR, dG, dB, dA);
-                  ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
+                ALPHA_BLEND_RGBA(sR, sG, sB, sA, dR, dG, dB, dA);
+                ASSEMBLE_RGBA(dst, dstbpp, dstfmt, dR, dG, dB, dA);
             }
             src++;
             dst += dstbpp;
         },
         width);
-        /* *INDENT-ON* */ /* clang-format on */
+        /* *INDENT-ON* */ // clang-format on
         src += srcskip;
         dst += dstskip;
     }
@@ -517,24 +517,37 @@ static const SDL_BlitFunc one_blitkey[] = {
 SDL_BlitFunc SDL_CalculateBlit1(SDL_Surface *surface)
 {
     int which;
-    SDL_PixelFormat *dstfmt;
 
-    dstfmt = surface->map->dst->format;
-    if (dstfmt->BitsPerPixel < 8) {
+    if (SDL_BITSPERPIXEL(surface->map.info.dst_fmt->format) < 8) {
         which = 0;
     } else {
-        which = dstfmt->BytesPerPixel;
+        which = SDL_BYTESPERPIXEL(surface->map.info.dst_fmt->format);
     }
-    switch (surface->map->info.flags & ~SDL_COPY_RLE_MASK) {
+
+    switch (surface->map.info.flags & ~SDL_COPY_RLE_MASK) {
     case 0:
-        return one_blit[which];
+        if (which < SDL_arraysize(one_blit)) {
+            return one_blit[which];
+        }
+        break;
 
     case SDL_COPY_COLORKEY:
-        return one_blitkey[which];
+        if (which < SDL_arraysize(one_blitkey)) {
+            return one_blitkey[which];
+        }
+        break;
 
-    case SDL_COPY_COLORKEY | SDL_COPY_BLEND:  /* this is not super-robust but handles a specific case we found sdl12-compat. */
-        return (surface->map->info.a == 255) ? one_blitkey[which] : (SDL_BlitFunc)NULL;
+    case SDL_COPY_COLORKEY | SDL_COPY_BLEND:  // this is not super-robust but handles a specific case we found sdl12-compat.
+        if (surface->map.info.a == 255) {
+            if (which < SDL_arraysize(one_blitkey)) {
+                return one_blitkey[which];
+            }
+        } else {
+            return which >= 2 ? Blit1toNAlphaKey : (SDL_BlitFunc)NULL;
+        }
+        break;
 
+    case SDL_COPY_BLEND:
     case SDL_COPY_MODULATE_ALPHA | SDL_COPY_BLEND:
         /* Supporting 8bpp->8bpp alpha is doable but requires lots of
            tables which consume space and takes time to precompute,
@@ -547,6 +560,4 @@ SDL_BlitFunc SDL_CalculateBlit1(SDL_Surface *surface)
     return (SDL_BlitFunc)NULL;
 }
 
-#endif /* SDL_HAVE_BLIT_1 */
-
-/* vi: set ts=4 sw=4 expandtab: */
+#endif // SDL_HAVE_BLIT_1
