@@ -11,8 +11,9 @@
 */
 /* Simple test of power subsystem. */
 
-#include <stdio.h>
-#include "SDL.h"
+#include <SDL3/SDL.h>
+#include <SDL3/SDL_main.h>
+#include <SDL3/SDL_test.h>
 
 static void
 report_power(void)
@@ -21,7 +22,7 @@ report_power(void)
     const SDL_PowerState state = SDL_GetPowerInfo(&seconds, &percent);
     const char *statestr = NULL;
 
-    SDL_Log("SDL-reported power info...\n");
+    SDL_Log("SDL-reported power info...");
     switch (state) {
     case SDL_POWERSTATE_UNKNOWN:
         statestr = "Unknown";
@@ -43,34 +44,46 @@ report_power(void)
         break;
     }
 
-    SDL_Log("State: %s\n", statestr);
+    SDL_Log("State: %s", statestr);
 
     if (percent == -1) {
-        SDL_Log("Percent left: unknown\n");
+        SDL_Log("Percent left: unknown");
     } else {
-        SDL_Log("Percent left: %d%%\n", percent);
+        SDL_Log("Percent left: %d%%", percent);
     }
 
     if (seconds == -1) {
-        SDL_Log("Time left: unknown\n");
+        SDL_Log("Time left: unknown");
     } else {
-        SDL_Log("Time left: %d minutes, %d seconds\n", seconds / 60, seconds % 60);
+        SDL_Log("Time left: %d minutes, %d seconds", seconds / 60, seconds % 60);
     }
 }
 
 int main(int argc, char *argv[])
 {
-    /* Enable standard application logging */
-    SDL_LogSetPriority(SDL_LOG_CATEGORY_APPLICATION, SDL_LOG_PRIORITY_INFO);
+    SDLTest_CommonState *state;
 
-    if (SDL_Init(0) == -1) {
-        SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "SDL_Init() failed: %s\n", SDL_GetError());
+    /* Initialize test framework */
+    state = SDLTest_CommonCreateState(argv, 0);
+    if (!state) {
+        return 1;
+    }
+
+    /* Parse commandline */
+    if (!SDLTest_CommonDefaultArgs(state, argc, argv)) {
+        return 1;
+    }
+
+    if (!SDL_Init(0)) {
+        SDL_LogError(SDL_LOG_CATEGORY_APPLICATION, "SDL_Init() failed: %s", SDL_GetError());
         return 1;
     }
 
     report_power();
 
     SDL_Quit();
+    SDLTest_CommonDestroyState(state);
+
     return 0;
 }
 
